var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
var utils;
(function (utils) {
    /**
 * 事件
 */
    var EventHelper = /** @class */ (function () {
        function EventHelper() {
        }
        /**
         * 注册事件
         * @param name 事件名称
         * @param callback 回调函数
         * @param context 上下文
         */
        EventHelper.register = function (name, callback, context) {
            var observers = EventHelper.listeners[name];
            if (!observers) {
                EventHelper.listeners[name] = [];
            }
            EventHelper.listeners[name].push(new Observer(callback, context));
        };
        EventHelper.once = function (name, callback, context) {
            var observers = EventHelper.listeners[name];
            if (!observers) {
                EventHelper.listeners[name] = [];
                observers = EventHelper.listeners[name];
            }
            var has = false;
            var length = observers.length;
            for (var i = 0; i < length; i++) {
                var observer = observers[i];
                if (observer.compar(context)) {
                    has = true;
                    break;
                }
            }
            if (!has) {
                EventHelper.listeners[name].push(new Observer(callback, context, true));
            }
        };
        /**
         * 移除事件
         * @param name 事件名称
         * @param callback 回调函数
         * @param context 上下文
         */
        EventHelper.remove = function (name, callback, context) {
            var observers = EventHelper.listeners[name];
            if (!observers)
                return;
            var length = observers.length;
            for (var i = 0; i < length; i++) {
                var observer = observers[i];
                if (observer.compar(context)) {
                    observers.splice(i, 1);
                    break;
                }
            }
            if (observers.length == 0) {
                delete EventHelper.listeners[name];
            }
        };
        /**
         * 发送事件
         * @param name 事件名称
         */
        EventHelper.fire = function (name) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            var observers = EventHelper.listeners[name];
            if (!observers)
                return;
            var length = observers.length;
            for (var i = 0; i < length; i++) {
                var observer = observers[i];
                observer.notify.apply(observer, __spreadArrays([name], args));
                if (observer.once) {
                    EventHelper.remove(name, observer.callback, observer.context);
                }
            }
        };
        /** 监听数组 */
        EventHelper.listeners = {};
        return EventHelper;
    }());
    utils.EventHelper = EventHelper;
    /**
     * 观察者
     */
    var Observer = /** @class */ (function () {
        function Observer(callback, context, once) {
            if (once === void 0) { once = false; }
            /** 回调函数 */
            this.callback = null;
            /** 上下文 */
            this.context = null;
            this.once = false;
            var self = this;
            self.callback = callback;
            self.context = context;
            self.once = once;
        }
        /**
         * 发送通知
         * @param args 不定参数
         */
        Observer.prototype.notify = function () {
            var _a;
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            var self = this;
            (_a = self.callback).call.apply(_a, __spreadArrays([self.context], args));
        };
        /**
         * 上下文比较
         * @param context 上下文
         */
        Observer.prototype.compar = function (context) {
            return context == this.context;
        };
        return Observer;
    }());
})(utils || (utils = {}));
//# sourceMappingURL=EventHelper.js.map