// TypeScript file
var ShowLayer;
(function (ShowLayer) {
    ShowLayer[ShowLayer["Game"] = 0] = "Game";
    ShowLayer[ShowLayer["Tip"] = 1] = "Tip";
    ShowLayer[ShowLayer["UI"] = 2] = "UI";
})(ShowLayer || (ShowLayer = {}));
var PageManager = /** @class */ (function () {
    function PageManager() {
        this.pageDic = new Dictionary();
        this.uiLayerStack = new Array();
    }
    Object.defineProperty(PageManager, "MaskArea", {
        get: function () {
            if (!this.instance) {
                throw new Error("请初始化管理器之后再使用");
            }
            return this.instance.maskArea;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(PageManager, "DialogMask", {
        get: function () {
            if (!this.instance) {
                throw new Error("请初始化管理器之后再使用");
            }
            return this.instance.dialogMask;
        },
        enumerable: false,
        configurable: true
    });
    PageManager.init = function (gameLayer, tipLayer, uiLayer, stage) {
        if (this.instance == null) {
            this.instance = new PageManager();
            this.instance.stage = stage;
            this.instance.gameLayer = gameLayer;
            this.instance.uiLayer = uiLayer;
            this.instance.uiLayer.autoSize = true;
            this.instance.tipLayer = tipLayer;
            // Laya.DebugPanel.init(false,)
            if (9 / 16 > Laya.Browser.clientWidth / Laya.Browser.clientHeight) {
                console.log("this a  full screen device");
                Global.IsFullScreen = true;
                Global.offsetY = (Laya.stage.height - 1334) / 2;
            }
            else if (9 / 16 < Laya.Browser.clientWidth / Laya.Browser.clientHeight) {
                this.instance.uiLayer.scale(Laya.stage.height / 1334, Laya.stage.height / 1334);
            }
            this.instance.stage.on(Laya.Event.RESIZE, this.instance.onResize, this);
            this.instance.stage.addChild(gameLayer);
            this.instance.stage.addChild(uiLayer);
            this.instance.stage.addChild(tipLayer);
            this.instance.uiLayer.centerX = 0;
            this.instance.uiLayer.centerY = 0;
            console.log(this.instance.uiLayer.y);
            this.instance.tipLayer.addChild(this.instance.maskArea = new view.MaskArea());
            this.instance.tipLayer.addChild(this.instance.dialogMask = new view.MaskArea());
            PageManager.MaskArea.DeActiveSelf();
            PageManager.DialogMask.DeActiveSelf();
            if (gameLayer.numChildren <= 0) {
                gameLayer.visible = false;
            }
            this.instance.tipLayer = tipLayer;
            if (tipLayer.numChildren <= 0) {
                tipLayer.visible = false;
            }
            this.instance.uiLayer = uiLayer;
            if (uiLayer.numChildren <= 0) {
                uiLayer.visible = false;
            }
        }
        return true;
    };
    PageManager.prototype.onResize = function () {
    };
    PageManager.getInstance = function () {
        return this.instance == null ? null : this.instance;
    };
    PageManager.prototype.get = function (key) {
        var page = this.pageDic.getValue(key);
        if (!page)
            page = this.createNewPage(key);
        return page;
    };
    PageManager.prototype.set = function (key, page) {
        this.pageDic.setValue(key, page);
    };
    PageManager.prototype.showTip = function (tip, args) {
        if (args === void 0) { args = { popWindow: false }; }
        this.tipLayer.visible = true;
        this.tipLayer.addChild(tip);
        tip.y = 550 + Global.offsetY;
        tip.showPage(args.data);
    };
    PageManager.prototype.hideTip = function (tip) {
        var _this = this;
        tip.hidePage().then(function () {
            _this.tipLayer.removeChild(tip);
        });
    };
    PageManager.prototype.showPage = function (name, args) {
        var _this = this;
        if (args === void 0) { args = { popWindow: false }; }
        var page = this.get(name);
        if (page == null)
            return;
        var currentPage = this.currentPage;
        currentPage && !args.popWindow && currentPage && currentPage.hidePage() && currentPage.hidePage().then(function () { return _this.uiLayer.removeChild(currentPage); });
        this.uiLayer.addChild(page);
        this.uiLayer.visible = true;
        this.uiLayerStack.push(page);
        page.showPage(args.data).then(function () { return _this.currentPage = page; });
    };
    PageManager.prototype.back = function (target) {
        if (target === void 0) { target = ""; }
        if (!target || target == "") {
            this.hidePage(this.uiLayerStack.pop().pageName, ShowLayer.UI, true);
            this.currentPage = this.uiLayerStack[this.uiLayerStack.length - 1];
            console.log(this.uiLayer.numChildren);
            if (!this.currentPage.parent)
                this.uiLayer.addChild(this.currentPage);
            this.currentPage.showPage();
        }
        else {
            var page = this.uiLayerStack.pop();
            this.hidePage(page.pageName, ShowLayer.UI);
            this.currentPage = this.uiLayerStack[this.uiLayerStack.length - 1];
            if (this.currentPage.pageName !== target) {
                this.back(target);
            }
            else {
                this.uiLayer.addChild(this.currentPage);
                this.currentPage.showPage();
            }
        }
    };
    PageManager.prototype.hidePage = function (name, layer, destroy) {
        var _this = this;
        if (destroy === void 0) { destroy = false; }
        var page = this.get(name);
        if (page == null) {
            return;
        }
        switch (layer) {
            case ShowLayer.Game:
                page.hidePage().then(function () {
                    _this.gameLayer.removeChild(page);
                    if (_this.gameLayer.numChildren <= 0) {
                        _this.gameLayer.visible = false;
                    }
                });
                break;
            case ShowLayer.Tip:
                page.hidePage().then(function () {
                    _this.tipLayer.removeChild(page);
                    if (_this.tipLayer.numChildren <= 0) {
                        _this.tipLayer.visible = false;
                    }
                });
                break;
            case ShowLayer.UI:
                page.hidePage().then(function () {
                    _this.uiLayer.removeChild(page);
                    if (_this.uiLayer.numChildren <= 0) {
                        _this.uiLayer.visible = false;
                    }
                });
                break;
        }
    };
    PageManager.prototype.createNewPage = function (name) {
        var page;
        switch (name) {
            case Global.Pages.MainMenu:
                page = new view.GameMainMenuPage();
                controller.EntryController.StartMenuController = new controller.StartMenuController(page);
                break;
            case Global.Pages.RankPage:
                page = new view.GameRankPage();
                controller.EntryController.RankPageController = new controller.RankPageController(page);
                break;
            case Global.Pages.MainGame:
                page = new view.GameMainPage();
                controller.EntryController.GamePageController = new controller.GamePageController(page);
                break;
            case Global.Pages.ResultEndPage:
                page = new view.GameResultEndPage();
                controller.EntryController.ResultEndController = new controller.ResultEndController(page);
                break;
            case Global.Pages.PropsPage:
                page = new view.PropsPage();
                break;
            case Global.Pages.DiaMondPage:
                page = new view.DiamondPage();
                break;
            case Global.Pages.LuckyDrawPage:
                page = new view.LuckyDrawPage();
                controller.EntryController.LuckyDrawController = new controller.LuckyDrawController(page);
                break;
            default:
                page = null;
                break;
        }
        if (page != null) {
            this.set(name, page);
        }
        return page;
    };
    PageManager.prototype.drawBackGround = function () {
        if (9 / 16 > Laya.stage.width / Laya.stage.height) {
            this.uiLayer.y += 130;
        }
    };
    return PageManager;
}());
//# sourceMappingURL=PageManager.js.map